#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define DIM_LIMIT 512

// macro di utilità per controllare errori
#define CHECK_EQ(X, val, str)	\
    if ((X)==val) {		\
      perror(#str);		\
      exit(EXIT_FAILURE);	\
    }
#define CHECK_NEQ(X, val, str)	\
    if ((X)!=val) {		\
      perror(#str);		\
      exit(EXIT_FAILURE);	\
    }


// tipo della funzione che uso per fare il confronto
typedef int (*F_t)(const void*, const void*, size_t);

int confronta(F_t cmp, const void *s1, const void *s2, size_t n) {
    return cmp(s1,s2,n);
}

int main(int argc, char *argv[]) {

    if (argc != 2) {
	fprintf(stderr, "use: %s <dim>\n", argv[0]);
	return EXIT_FAILURE;
    }
    long dim = strtol(argv[1], NULL, 10);
    if (dim > DIM_LIMIT) {
	fprintf(stderr,"dim (%ld) è troppo grande, deve essere <=%d\n", dim, DIM_LIMIT);
	return EXIT_FAILURE;
    }

    float *mat = NULL;
    // alloco la matrice come un singolo vettore
    CHECK_EQ(mat = malloc(dim*dim*sizeof(float)), NULL, "malloc");

    // riempio la matrice
    for(long i=0;i<dim;++i)
	for(long j=0;j<dim;++j)
	    mat[i*dim + j] = (i+j)/2.0;

    FILE *fp1;
    CHECK_EQ(fp1 = fopen("./mat_dump.dat", "wb"), NULL, "fopen");

#if 0 // write singola
    CHECK_NEQ(fwrite(&mat[0], sizeof(float), dim*dim, fp1), (dim*dim), "fwrite");
#else // alternativamente 
    for(long i=0;i<dim;++i)
	for(long j=0;j<dim;++j) 
	    CHECK_NEQ(fwrite(&mat[i*dim+j], sizeof(float), 1, fp1), 1, "fwrite");    
#endif
    fclose(fp1);

    // dump testuale
    CHECK_EQ(fp1 = fopen("./mat_dump.txt", "w"), NULL, "fopen");
    for(long i=0;i<dim;++i)
	for(long j=0;j<dim;++j) {
	    if (fprintf(fp1, "%f\n", mat[i*dim+j])<0) {
		perror("fprintf");
		return EXIT_FAILURE;
	    }
	}
    fclose(fp1);  


    float *mat2 = NULL;
    CHECK_EQ(mat2 = malloc(dim*dim*sizeof(float)), NULL, "malloc");
    CHECK_EQ(fp1=fopen("./mat_dump.dat", "rb"), NULL, "fopen");

#if 1 // read singola
    CHECK_NEQ(fread(&mat2[0], sizeof(float), dim*dim, fp1),(dim*dim), "fread");
#else // alternativamente
    for(long i=0;i<dim;++i)
	for(long j=0;j<dim;++j)
	    CHECK_NEQ(fread(&mat2[i*dim+j], sizeof(float), 1, fp1), 1, "fread");
#endif
    fclose(fp1);

    if (confronta(memcmp, mat, mat2, dim*dim*sizeof(float)) != 0) {
	fprintf(stderr, "Le due matrici non corrispondono\n");
	return EXIT_FAILURE;
    }
    fprintf(stdout, "Le due matrici sono uguali\n");
    free(mat);
    free(mat2);
    return 0;
}




